--[[
Name: FuBar_DungeonDiFu
Revision: $Rev: 35 $
Author(s): Myrathi (myrathi@gmail.com)
Website: http://www.wowace.com/addons/fubar-dungeondifu/
SVN: svn://svn.wowace.com/wow/fubar-dungeondifu/mainline/trunk
Description: Provides more visible access to dungeon and raid difficulty settings via FuBar.
Dependencies: (optional) FuBar, (optional) Ace2, (optional) DewdropLib, (optional) TabletLib, (optional) FuBarPlugin-2.0
Id: $Id: DungeonDiFu.lua 35 2009-10-07 02:04:54Z myrathi2 $
]]

local Tab = AceLibrary("Tablet-2.0")
local L = AceLibrary("AceLocale-2.2"):new("FuBar_DungeonDiFu")
local DF = AceLibrary("Deformat-2.0")

DungeonDiFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "FuBarPlugin-2.0")

DungeonDiFu:RegisterDB("DungeonDiFuDB")
DungeonDiFu:RegisterDefaults('profile', 
	{
		ShowHint	= true,
		ShowPrefix	= true,
		AlwaysShowBothDiffs = false
	}
)

DungeonDiFu.iconPaths = {
	["party"] = {
		[1] = "Interface\\ICONS\\INV_Misc_Platnumdisks.blp",       -- 5 (normal) [Stone Keeper Shards]
		[2] = "Interface\\ICONS\\Spell_Holy_ProclaimChampion.blp", -- 5 (heroic) [Emblem of Heroism]
	},
	["raid"] = {
		[1] = "Interface\\ICONS\\Spell_Holy_ProclaimChampion.blp", -- 10 (normal)  [Emblem of Heroism]
		[2] = "Interface\\ICONS\\Spell_Holy_ChampionsGrace.blp",   -- 10 (heroic)  [Emblem of Conquest]
		[3] = "Interface\\ICONS\\Spell_Holy_SummonChampion.blp",   -- 25 (normal)  [Emblem of Triumph]
		[4] = "Interface\\ICONS\\INV_Misc_Trophy_Argent.blp",      -- 25 (heroic)  [Trophy of the Crusade]
	}
};

DungeonDiFu.version = GetAddOnMetadata("FuBar_DungeonDiFu", "Version").." (r"..("$Rev: 35 $"):match("%d+")..")"
DungeonDiFu.date = string.sub("$Date: 2009-10-07 02:04:54 +0000 (Wed, 07 Oct 2009) $", 8, 17)
DungeonDiFu.hasIcon = DungeonDiFu.iconPaths["party"][1]
DungeonDiFu.hasNoColor = true
DungeonDiFu.canHideText = true
DungeonDiFu.clickableTooltip = true
DungeonDiFu.cannotDetachTooltip = true
DungeonDiFu.defaultPosition = 'LEFT'

--============================================================================
DungeonDiFu.OnMenuRequest = 
{
	handler = DungeonDiFu,
	type = 'group',
	args = 
	{
		showHint =
		{
			type = "toggle",
			order = 101,
			name = L["Show Hint"],
			desc = L["Show hint at the bottom of the tooltip."],
			get = 'CanShowHint',
			set = 'SetShowHint'
		},
		showPrefix =
		{
			type = "toggle",
			order = 102,
			name = L["Show Prefix"],
			desc = L["Show text prefix before difficulty."],
			get = 'CanShowPrefix',
			set = 'SetShowPrefix'
		},
		alwaysShowBothDiffs =
		{
			type = "toggle",
			order = 103,
			name = L["Always Show Both"],
			desc = L["Always show both difficulty settings, regardless of party/raid status."],
			get = 'CanAlwaysShowBothDiffs',
			set = 'SetAlwaysShowBothDiffs'
		},
	},
}

function DungeonDiFu:CanShowHint()
	return self.db.profile.ShowHint
end

function DungeonDiFu:SetShowHint()
	self.db.profile.ShowHint = not self.db.profile.ShowHint
	self:UpdateTooltip()
end

function DungeonDiFu:CanShowPrefix()
	return self.db.profile.ShowPrefix
end

function DungeonDiFu:SetShowPrefix()
	self.db.profile.ShowPrefix = not self.db.profile.ShowPrefix
	self:UpdateText()
end

function DungeonDiFu:CanAlwaysShowBothDiffs()
	return self.db.profile.AlwaysShowBothDiffs
end

function DungeonDiFu:SetAlwaysShowBothDiffs()
	self.db.profile.AlwaysShowBothDiffs = not self.db.profile.AlwaysShowBothDiffs
	self:UpdateText()
end

--============================================================================
function DungeonDiFu:OnInitialize()
end

function DungeonDiFu:OnEnable()
    self:RegisterEvent("PARTY_MEMBERS_CHANGED", "UpdateDisplay")
    self:RegisterEvent("CHAT_MSG_SYSTEM")
end

function DungeonDiFu:OnDisable()
end

--============================================================================
function DungeonDiFu:CHAT_MSG_SYSTEM(message)
  --ERR_DUNGEON_DIFFICULTY_CHANGED_S = "Dungeon Difficulty set to %s."; --30200
  --ERR_RAID_DIFFICULTY_CHANGED_S = "Raid Difficulty set to %s."; --30200
  -- Doesn't matter which we catch.
	local diff = DF(message, ERR_DUNGEON_DIFFICULTY_CHANGED_S) or DF(message, ERR_RAID_DIFFICULTY_CHANGED_S)
	if (diff ~= nil) then
		self:UpdateDisplay()
	end
end

--============================================================================
function DungeonDiFu:OnTextUpdate()
	local isParty, isRaid, ddiff, rdiff = self:GetPartyRaidInfo()
	self:SetIcon(self.iconPaths[isRaid and "raid" or "party"][isRaid and rdiff or ddiff])

	local text, isSolo = "", not (isParty or isRaid)
	if (self.db.profile.ShowPrefix) then
		text = ((isRaid and RAID) or (isParty and PARTY) or SOLO) .. ": "
	end
	local forceBoth = self.db.profile.AlwaysShowBothDiffs
	local showParty = forceBoth or not isRaid
	local showRaid = forceBoth or isSolo or isRaid
	if (showParty) then text = text .. _G["DUNGEON_DIFFICULTY"..ddiff] end
	if (forceBoth or isSolo) then text = text .. " / " end
	if (showRaid) then text = text .. _G["RAID_DIFFICULTY"..rdiff] end
	self:SetText(text)
end

--============================================================================
function DungeonDiFu:OnTooltipUpdate()
	local cantChange = not self:CanAlterDiff()
	local _, _, ddiff, rdiff = self:GetPartyRaidInfo()
	Tab:SetTitle(DUNGEON_DIFFICULTY)
	local cat = Tab:AddCategory('text', PARTY..":")
	for i = 1, #self.iconPaths["party"] do
		cat:AddLine(
			'text', _G["DUNGEON_DIFFICULTY"..i],
			'textR', cantChange and 0.6 or nil,
			'textG', cantChange and 0.6 or nil,
			'textB', cantChange and 0.6 or nil,
			'hasCheck', true,
			'checked', (i == ddiff),
			'func', self:CanAlterDiff() and 'OnClickDungeonDifficulty' or nil,
			'arg1', self,
			'arg2', i
		)
	end
	cat = Tab:AddCategory('text', RAID..":")
	for i = 1, #self.iconPaths["raid"] do
		cat:AddLine(
			'text', _G["RAID_DIFFICULTY"..i],
			'textR', cantChange and 0.6 or nil,
			'textG', cantChange and 0.6 or nil,
			'textB', cantChange and 0.6 or nil,
			'hasCheck', true,
			'checked', (i == rdiff),
			'func', self:CanAlterDiff() and 'OnClickRaidDifficulty' or nil,
			'arg1', self,
			'arg2', i
		)
	end

	if self.db.profile.ShowHint then
		Tab:SetHint(L["TOOLTIP_HINT"])
	end
end

function DungeonDiFu:CanAlterDiff()
	-- You can alter dungeon difficulty if you're solo (GetNum* == 0) or
	-- are the party or raid leader (assistant can't).
	return (GetNumPartyMembers() + GetNumRaidMembers() == 0) or IsPartyLeader()
end

function DungeonDiFu:GetPartyRaidInfo()
	-- GetNumPartyMembers() - 0-4 members, excluding you (0 = solo)
	-- GetNumRaidMembers() - raid members, including you.
	-- GetDungeonDifficulty() - 1-2 (5, 5H)
	-- GetRaidDifficulty() - 1-4 (10, 25, 10H, 25H)
	return (GetNumPartyMembers() > 0), (GetNumRaidMembers() > 0), GetDungeonDifficulty(), GetRaidDifficulty()
end

function DungeonDiFu:OnClickDungeonDifficulty(diff)
	SetDungeonDifficulty(diff)
end

function DungeonDiFu:OnClickRaidDifficulty(diff)
	SetRaidDifficulty(diff)
end

--============================================================================
